      SUBROUTINE CYCLE(L)

      INCLUDE  'type.inc'
      INCLUDE  'dimen.inc'
      INCLUDE  'miscel.inc'
      INCLUDE  'vecs.inc'
      INCLUDE  'draw.inc'

C***********************************************************************
C  This subroutine is performing the 1-D integration of the SW
C  conservation laws.
C***********************************************************************
C  Note on indexing:
C  Since loops here are 1-D, we use the following convention concerning
C        the indexing of variables centered at grid cells or grid points
C        (which correspond to cell faces in the 3-D configuration).
C  INDEX  I  --  Denotes cell-interfaces
C  INDEX  K  --  Denotes cell-centers
C***********************************************************************

      INTEGER*4 L,I,K,N,KDTmin
      INTEGER*4 ISIGN        
      REAL*8    DTmin
      REAL*8    ROold(LDIM),ROUold(LDIM)
      REAL*8    RO_implicit,Sf_implicit,So_midcell
      logical now
      data    now/.false./

C***********************************************************************

      now=t.eq.0.d0

C***********************************************************************
C***********************************************************************
C   STEP  1.  SLOPES OF FLOW VARIABLES AT   T(N+1)
C***********************************************************************
C***********************************************************************
C         NOTE:  SLOPES ARE HERE UPDATED TO NEW TIME LEVEL, INSTEAD OF
C                AT THE END OF FORMER CYCLE.  THIS WAY VALUES OF SLOPES
C                AT OUTPUT (PRINT OR WRITE) ARE OLD ONES, I>E>, THOSE
C                WHICH SERVED IN THE GRP FOR THE LAST INTEGRATION STEP.
C***********************************************************************
C***********************************************************************

C=======================================================================
C   1.1   New slopes  DU(K),  DRO(K)  (and slope monotonization)
C=======================================================================

                     IF(GODUNOV) THEN
      DO K=2,LL         ! GODUNOV Scheme, zero slopes
         DU (K) = 0.d0
         DRO(K) = 0.d0
      END DO
                     ELSE  ! GRP Scheme

               IF(KEYslope.EQ.'CID') THEN
      DO K=2,LL         ! GRP/CID (Cell-Interface Differencing)
         I=K
         DU (K) = ( Ux(I+1) -  Ux(I)) + DT*( DUDTx(I+1) -  DUDTx(I))
         DRO(K) = (ROx(I+1) - ROx(I)) + DT*(DRODTx(I+1) - DRODTx(I))
      END DO
               ELSEIF(KEYslope.EQ.'CD ') THEN
      DO K=2,LL         ! GRP/CD (Centered Differencing of cell values)
!        DU (K) = (U (K+1) - U  (K))/2.d0
!        DRO(K) = (RO(K+1) - RO (K))/2.d0
         DU (K) = (U (K+1) - U  (K))
         DRO(K) = (RO(K+1) - RO (K))
      END DO
               ELSE
      GOTO 7001
               ENDIF ! End KEYslope

      CALL MONOTON(L,1,U , DU)
      CALL MONOTON(L,2,RO,DRO)

                     ENDIF ! End GODUNOV/GRP Schemes

C=======================================================================
C   1.2    SET SLOPES IN BOUNDARY AND NEXT-TO-BOUNDARY CELLS TO ZERO
C=======================================================================
!        DU  ( 2)=0.d0
!        DRO ( 2)=0.d0
!        DU  (LL)=0.d0
!        DRO (LL)=0.d0

                  ISIGN = +1 ! Transmitting BC
                  ISIGN = -1 ! Rigid wall   BC

! take account of the slope
         DU  ( 2)=(1+ISIGN)/2.d0*DU ( 2)
         DRO ( 2)=(1+ISIGN)/2.d0*DRO( 2) - (Z(2)-Z( 1))*2.d0
         DU  (LL)=(1+ISIGN)/2.d0*DU (LL)
         DRO (LL)=(1+ISIGN)/2.d0*DRO(LL) - (Z(L)-Z(LL))*2.d0

C***********************************************************************
C***********************************************************************
C   STEP  2.  PREPARE CELL-INTERFACE VALUES FOR SOLUTION OF  RP & GRP
C***********************************************************************
C***********************************************************************

C=======================================================================
C     2.1    UL,ROL,   UR,ROR,   CL,GL,  CR,GR
C=======================================================================

      DO I=2,L
         K=I

         UR  (I) = U  (K  )-DU  (K  )/2.d0
         ROR (I) = RO (K  )-DRO (K  )/2.d0
 
         UL  (I) = U  (K-1)+DU  (K-1)/2.d0
         ROL (I) = RO (K-1)+DRO (K-1)/2.d0
      END DO

!     UL  (2) = UR  (2)
!     ROL (2) = ROR (2)
!     UR  (L) = UL  (L)
!     ROR (L) = ROL (L)
 
! It is preferrable to call boundary once more 
! to make sure that transmitting BC are included 

      CALL BOUNDARY(L)

      IF(KEYfriction.EQ.1) THEN
        DO I=2,L
          SfR(I) = eta2*UR(I)*abs(UR(I))*ROR(I)**(-4.d0/3.d0)
          SfL(I) = eta2*UL(I)*abs(UL(I))*ROL(I)**(-4.d0/3.d0)
          Sf (I) = (SfL(I)+SfR(I))/2.d0
        END DO
      END IF

      DO I=2,L
         CL  (I) = SQRT(GG*ROL(I))
         CR  (I) = SQRT(GG*ROR(I))
      END DO

      DO I=2,L
         GL  (I) = ROL(I)*CL(I)
         GR  (I) = ROR(I)*CR(I)
      END DO

C=======================================================================
C     2.2  GRADIENTS AT CELL-INTERFACES
C          Note: Here we define DXSIL(I)/DXSIR(I) with the corresponding
C                point density    ROL(I)/  ROR(I), unlike former GRP
C                versions where RO(K) of the adjacent cell was used.
C                The reasoning for this choice is Lagrangian gradients
C                are obtained from the Eulerian ones by the correct
C                local transformation, e.g.,
C                DUDXSIR(I)=DU(K)/(DX(K)*ROR(I)), with K=I.
C=======================================================================

      DO K=2,LL
         I = K
!-----------------------------------------------------------------------
! Note:  In LEGS and in Cubic we used cell density RO(K)
!-----------------------------------------------------------------------
!        DXSIR(I  ) = ROR(I  )* DX(K)
!        DXSIL(I+1) = ROL(I+1)* DX(K)
         DXSIR(I  ) = ROR(I  )*bdx(K) 
         DXSIL(I+1) = ROL(I+1)*bdx(K)
      END DO
         DXSIL(2)=DXSIR(2)
         DXSIR(L)=DXSIL(L)

      DO I=2,L
         K=I
         DUDXIL (I)=DU (K-1)/DXSIL(I)
         DRODXIL(I)=DRO(K-1)/DXSIL(I)
         DUDXIR (I)=DU (K  )/DXSIR(I)
         DRODXIR(I)=DRO(K  )/DXSIR(I)
      END DO

C=======================================================================
C     2.3  IMPLEMENT BOUNDARY CONDITIONS AT ALL POINTS BY KIND(K)
C=======================================================================

      CALL BOUNDARY(L)

C***********************************************************************
C***********************************************************************
C   STEP  3.  SOLVE GENERALIZED RIEMANN PROBLEMS AND EVALUATE FLUXES
C***********************************************************************
C***********************************************************************
C     
C=======================================================================
C     3.1  GENERALIZED RIEMANN PROBLEMS (ACOUSTIC & EXACT)
C=======================================================================

      CALL GRP(L)

C=======================================================================
C     3.2  FLUXES AT  T(N+0)  AND THEIR FIRST TIME DERIVATIVE
C=======================================================================

      CALL FLUXES(L)

C***********************************************************************
C***********************************************************************
C   STEP  4.  CONSERVATION LAWS.  UPDATE FLOW VARIABLES  T(N) --> T(N+1)
C***********************************************************************
C***********************************************************************

C=======================================================================
C   4.1 
C=======================================================================

      DO K=2,LL
          ROold(K) =  RO(K)
         ROUold(K) = ROU(K)
      END DO

C=======================================================================
C   4.2   MASS AND MOENTUM RO(K),ROU(K)  at  T(N+1)
C=======================================================================

      DO K=2,LL
         I=K
C***********************************************************************
!         RO(K)  =  ROold(K) - DT*(FLUX(I+1,1)-FLUX(I,1))/ DX(K)
!        ROU(K)  = ROUold(K) - DT*(FLUX(I+1,2)-FLUX(I,2))/ DX(K)
C***********************************************************************
          RO(K)  =  ROold(K) - DT*(FLUX(I+1,1)-FLUX(I,1))/BDX(K)
          RO_implicit = (RO(K)+ROold(K))/2.d0
          So_midcell  = (So(I+1)+ So(I))/2.d0

       IF (KEYfriction.EQ.0) Sf_implicit =0.0d0
       IF (KEYfriction.EQ.1) Sf_implicit =
     1          eta2*U (I)*abs(U (I))*RO_implicit**(-4.d0/3.d0)

         ROU(K) = ROUold(K) - DT*(FLUX(I+1,2)-FLUX(I,2))/BDX(K)
     1                      - DT*(GLUX(I+1,2)-GLUX(I,2))/ DX(K)
     2                      + DT*GG*RO_implicit *So_midcell
     3                      - DT*GG*RO_implicit *Sf_implicit
C***********************************************************************
! The source terms appearing in column matrix H eq. (1.2) in my notes
! are computed with the implicit ro. However, if KEYfriction is not 0
! and the Manning model is used, iteration to find the implicit velocity
! are needed.
! The bottom slope of the computational cell at its middle is taken as
! the average of the slopes at its boundary.
C***********************************************************************
      END DO

C=======================================================================
C   4.3 
C=======================================================================

      AMTOT  = 0
      ROUTOT = 0
      DO K=2,LL
!        AMTOT  =  AMTOT +  RO(K)* DX(K)
!       ROUTOT  = ROUTOT + ROU(K)* DX(K)
         AMTOT  =  AMTOT +  RO(K)*BDX(K)
        ROUTOT  = ROUTOT + ROU(K)*BDX(K)
      END DO

C=======================================================================
C   4.4   Velocity  U(K)  at  T(N+1)
C=======================================================================

      DO K=2,LL
         U(K) = ROU(K)/RO(K)
      END DO

C***********************************************************************
C***********************************************************************
C   5.   NEXT TIME STEP.  DTmin  IS THE MINIMAL ROW/COLUMN TIME STEP
C***********************************************************************
C***********************************************************************

      DO K=2,LL
         C(K)=SQRT(GG*RO(K))
      END DO

C=======================================================================
C     5.1  CELL TIME STEP  DTNEXT_CO(K)  FOR THIS  ROW/COLUMN
C=======================================================================

      DO K=2,LL
         I=K
         DTNEXT_C(K)=DX(K)/MAX(ABS(UW(5,I)),ABS(UW(1,I+1)),
     1                         ABS(U(K))+C(K))
      END DO

C=======================================================================
C     5.2  THE MINIMAL TIME STEP  DTmin  FOR THIS  ROW/COLUMN
C=======================================================================
      DTmin=1.d+77
      DO K=2,LL
         IF(DTmin.GT.DTNEXT_C(K)) THEN
            DTmin=DTNEXT_C(K)
            KDTmin=K
         ENDIF
      END DO
      DTNEXT = DTmin
      KDT    = KDTmin
C***********************************************************************
C***********************************************************************

c     print*,'END CYCLE'
      RETURN

C***********************************************************************
C     ********   ERROR  MESSAGES   ********************
C***********************************************************************
 7001 CONTINUE
      PRINT 7101,KEYslope
 7101 FORMAT(/'Unknown slope option.  KEYslope=',A3/)
      CALL ENDRUN(7001)
      END




