      SUBROUTINE  DAM_BREAK    ! DAM BREAK (High on left:  ROLI > RORI)

                include  'type.inc'
                include  'dimen.inc'
                include  'init.inc'
                include  'miscel.inc'
                include  'draw.inc'

C***********************************************************************
C  An axact solution for the dam break problem is calculated based on
C  the flow being a solution to a Riemann problem with a left-facing
C  CRW and a right-facing shock wave, but no contact discontinuity.
C  The two variables U, RO, are calculated for t=TMAX as function of the
C  spatial coordinate.
C***********************************************************************
      INTEGER      K
      INTEGER*4    I,N,NMAX
      REAL*8       DXX
      REAL*8       U,RO,C
      REAL*8       ULsave,ROLsave,URsave,RORsave
      REAL*8       Ustar,ROstar,Cstar
      REAL*8       XX,Xmid,X1,X2,X3,Xmean
      REAL*8       Uprime_shockR,Uprime_rarefL,UshockR,UrarefL
      REAL*8       EPS,UMIDA,DU,DUDROL,DUDROR,DUDRO,USR
      LOGICAL      REFLECT
      LOGICAL      DRY
C***********************************************************************
      COMMON/EXACT/ROexact(LDIM),Uexact(LDIM)   
      REAL*8       ROexact      ,Uexact   
C***********************************************************************
      UshockR(RO)       = URI+Aphalf*(RO-RORI)*SQRT(1/RO+1/RORI)
      UrarefL(RO)       = ULI-SQRT(8*A)*(SQRT(RO)-SQRT(ROLI))
      Uprime_shockR(RO) =  (Aphalf*SQRT(1/RO+1/RORI))
     1                   -(Aphalf*(RO-RORI))/(2*SQRT(1/RO+1/RORI)*RO**2)
      Uprime_rarefL(RO) = -SQRT(2*A/RO)
C***********************************************************************
      DATA  NMAX/  77 /
      DATA  EPS/1.d-10/
         REFLECT = .FALSE.
         DRY     = .TRUE.
         DRY     = .FALSE.
      if(.not.DRY) then	
C***********************************************************************
C  Solution of the Riemann Problem
C***********************************************************************
      IF(ROLI.LT.RORI) THEN ! Interchange directions
         ROLsave = ROLI
          ULsave =  ULI
         RORsave = RORI
          URsave =  URI
           ROLI  = RORsave
            ULI  =  URsave
           RORI  = ROLsave
            URI  =  ULsave
         REFLECT = .TRUE.
      ENDIF
      UMIDA=EPS*CLI
      DUDROL=-(CLI/ROLI)
      DUDROR= (CRI/RORI)
      RO=(ULI-URI+RORI*DUDROR-ROLI*DUDROL)/(  DUDROR-  DUDROL)
      U =(ROLI-RORI+URI/DUDROR-ULI/DUDROL)/(1/DUDROR-1/DUDROL)
      N=0
 11   N=N+1
      IF (N.GT.NMAX) GO TO 7001
      DU=UrarefL(RO)-UshockR(RO)
      IF (ABS(DU).LE.UMIDA) GO TO 10
      DUDRO=Uprime_rarefL(RO)-Uprime_shockR(RO)
      RO=RO-(DU/DUDRO)
      GO TO 11
 10   CONTINUE
      Ustar  = (UrarefL(RO)+UshockR(RO))/2
      ROstar = RO
      Cstar  = SQRT(2*A*RO)
      USR    = SQRT(A*ROstar*(1+ROstar/RORI))
      PRINT 12,N,ULI,ROLI,URI,RORI,Ustar,ROstar
 12   FORMAT(/'DAM_BREAK.  Iterations converged.   N=',I4/
     1        'ULI,ROLI,URI,RORI=',1P,4E13.4/
     2        'Ustar,ROstar     =',1P,2E13.4/)
C***********************************************************************
C  Calculate  RO(XX,Tmax), U(XX,Tmax)  and write on  fort.99
C***********************************************************************
      N      = 101
      N      =  51
C  x1,x2,x3 are special points for this problem
      Xmid=(Xmin+Xmax)/2
        X1=Xmid-         CLI *Tmax           
        X2=Xmid+(Ustar-Cstar)*Tmax
        X3=Xmid+         USR *Tmax
C***********************************************************************
      PRINT*,'DAM_BREAK.  WRITE SELF-SIMILAR PROFILE ON fort.99'
      PRINT*,'Xmin,Xmid,Xmax=',Xmin,Xmid,Xmax
      PRINT*,'X1,X2,X3=',X1,X2,X3
      PRINT*,'ROstar,Ustar,USR=',ROstar,Ustar,USR
      OPEN(99,FORM='FORMATTED',FILE='fort.99',STATUS='NEW')
      WRITE(99,102) ROLI,ULI,RORI,URI,N,Tmax
 102  FORMAT('# SELF-SIMILAR SOLUTION.'/
     1       '# ROLI,ULI=',1P,2E12.4/
     2       '# ROLR,ULR=',1P,2E12.4/
     3       '# N,Tmax=',I6,1P,E12.3/
     4       '# I,XX,U(I),RO(I) ='/
     5       '#                         ')
C***********************************************************************
C  Time for density and velocity distributions
C***********************************************************************
      
      DO I=1,N
         XX=Xmin+(I-1)*(X1-Xmin)/(N-1)
        RO=ROLI
         U= ULI
                 IF(.NOT.REFLECT) THEN
         PRINT 21,I,XX,U,RO
         WRITE(99,21)I,XX,U,RO
                 ELSE
         PRINT 21,I,Xmax-XX,U,RO
         WRITE(99,21)I,Xmax-XX,U,RO
                 ENDIF
      END DO
 21   FORMAT(I5,1P,3E12.5)

      DO I=N+1,2*N
         RO=ROLI-(I-(N+1))*(ROLI-ROstar)/(N-1)
         U= ULI-SQRT(8*A)*(SQRT(RO)-SQRT(ROLI))
         C=SQRT(2*A*RO)
         XX=Xmid+(U-C)*Tmax
                 IF(.NOT.REFLECT) THEN
         PRINT    21,I,XX,U,RO
         WRITE(99,21)I,XX,U,RO
                 ELSE
         PRINT    21,I,Xmax-XX,U,RO
         WRITE(99,21)I,Xmax-XX,U,RO
                 ENDIF
      END DO

      DO I=2*N+1,3*N
         XX=X2+(I-(2*N+1))*(X3-X2)/(N-1)
         RO=ROstar
         U=  Ustar
                 IF(.NOT.REFLECT) THEN
         PRINT    21,I,XX,U,RO
         WRITE(99,21)I,XX,U,RO
                 ELSE
         PRINT    21,I,Xmax-XX,U,RO
         WRITE(99,21)I,Xmax-XX,U,RO
                 ENDIF
      END DO
      DO I=3*N+1,4*N
         XX=X3+(I-(3*N+1))*(Xmax-X3)/(N-1)
         RO=RORI
         U=  URI
                 IF(.NOT.REFLECT) THEN
         PRINT    21,I,XX,U,RO
         WRITE(99,21)I,XX,U,RO
                 ELSE
         PRINT    21,I,Xmax-XX,U,RO
         WRITE(99,21)I,Xmax-XX,U,RO
                 ENDIF
      END DO

      IF(REFLECT) THEN ! Return to saved initial data
           ROLI  = ROLsave
            ULI  =  ULsave
           RORI  = RORsave
            URI  =  URsave
      ENDIF

C***********************************************************************
C  To compare the accuracy of the GRP scheme with other schemes the    *
C  exact solution in the COMPUTATIONAL CELLS has to be known           *
C***********************************************************************
!     At this stage X(I) is not yet defined. 
!     We assume evenly spaced spatial grid.
!     DO I=2,L
!        X(I)=Xmin+(I-2)*DXX
!     END DO
C***********************************************************************

      DXX=(Xmax-Xmin)/(LL-1)
      DO K=2,LL
        I=K
        Xmean = Xmin+(I-1.5d0)*DXX
        IF(Xmean.LE.X1                  ) THEN  ! Left  to the head charachteristic
          ROexact(K)=ROLI
           Uexact(K)= ULI
        END IF
        IF(Xmean.LE.X2 .AND. Xmean.GT.X1) THEN  ! Within the Rarefaction
C***********************************************************************
!       To find U and RO in this region we use the following:
!       U-C= (Xmean-Xmid)/TMAX
!       U-C= ULI-2.d0*(SQRT(2*A*RO)-SQRT(2*A*ROLI))-SQRT(2*A*RO)
C***********************************************************************
        C=(ULI+SQRT(8*A*ROLI)-(Xmean-Xmid)/TMAX)/3.d0
           Uexact(K)=  C     +(Xmean-Xmid)/TMAX
          ROexact(K)=C*C/2.d0/A
        END IF
        IF(Xmean.LE.X3 .AND. Xmean.GT.X2) THEN  ! Right to the tail charachteristic left to the shock
          ROexact(K)=ROstar
           Uexact(K)= Ustar
        END IF
        IF(                  Xmean.GT.X3) THEN  !                                  Right to the shock
          ROexact(K)=RORI
           Uexact(K)= URI
        END IF
      END DO

      end if    ! Not DRY

      if(     DRY) then
C***********************************************************************
C  The case of a dry bed on the right side of the dam has a different  *
C  solution see Zoppou and Roberts 'explicit schemes for Dam-Break     *
C  simulation' J. Hydraulic Eng. 11 2003                               *
C***********************************************************************
C  x1,x2,x3 are special points for this problem
        X1=Xmid-     CLI*Tmax
        X2=Xmid+2.d0*CLI*Tmax
        X3=X2
C***********************************************************************
      PRINT*,'DAM_BREAK.  WRITE SELF-SIMILAR PROFILE ON fort.99'
      PRINT*,'Xmin,Xmid,Xmax=',Xmin,Xmid,Xmax
      PRINT*,'X1,X2,X3=',X1,X2,X3
      OPEN(99,FORM='FORMATTED',FILE='fort.99',STATUS='NEW')
      WRITE(99,102) ROLI,ULI,RORI,URI,N,Tmax
C***********************************************************************
C  Time for density and velocity distributions
C***********************************************************************
      DO I=1,N
         XX=Xmin+(I-1)*(X1-Xmin)/(N-1)
        RO=ROLI
         U= ULI
                 IF(.NOT.REFLECT) THEN
         PRINT 21,I,XX,U,RO
         WRITE(99,21)I,XX,U,RO
                 ELSE
         PRINT 21,I,Xmax-XX,U,RO
         WRITE(99,21)I,Xmax-XX,U,RO
                 ENDIF
      END DO

      DO I=N+1,3*N
         XX =X1+(I-(N+0))*(X2-X1)/(2*N-0)
         RO=2.d0/9.d0/A*(SQRT(2.*A*ROLI)-(XX-XMID)/2.d0/TMAX)**2
         U =2.d0/3.d0  *(SQRT(2.*A*ROLI)+(XX-XMID)/     TMAX)
!        RO=ROLI-(I-(N+1))*(ROLI-0.0d0 )/(2*N-1)
!        U= ULI-SQRT(8*A)*(SQRT(RO)-SQRT(ROLI))
!        C=SQRT(2*A*RO)
!        XX=Xmid+(U-C)*Tmax
                 IF(.NOT.REFLECT) THEN
         PRINT    21,I,XX,U,RO
         WRITE(99,21)I,XX,U,RO
                 ELSE
         PRINT    21,I,Xmax-XX,U,RO
         WRITE(99,21)I,Xmax-XX,U,RO
                 ENDIF
      END DO

      DO I=3*N+1,4*N
         XX=X3+(I-(3*N+1))*(Xmax-X3)/(N-1)
         RO=0.0d0
         U =0.0d0
                 IF(.NOT.REFLECT) THEN
         PRINT    21,I,XX,U,RO
         WRITE(99,21)I,XX,U,RO
                 ELSE
         PRINT    21,I,Xmax-XX,U,RO
         WRITE(99,21)I,Xmax-XX,U,RO
                 ENDIF
      END DO

      end if  ! DRY
      RETURN
C***********************************************************************
C************* ERROR MESSAGES ******************************************
C***********************************************************************
 7001 CONTINUE
      PRINT 7101,I,N,DU,UMIDA,EPS,ROLI,ULI,RORI,URI,U,RO,DUDRO
 7101 FORMAT(/'DAM_BREAK.  NUMBER OF ITERATIONS EXCEEDED.'//
     1        'I,N,DU,UMIDA,EPS=',2I6,3E18.6//
     2        'ROLI,ULI,RORI,URI=',4E18.6//
     3        'U,RO,DUDRO=',3E18.6//)
      CALL ENDRUN(7001)

      END




