      SUBROUTINE DUCVAL (L)                                 
C***********************************************************************
C                  Rectangular channel                                 *
C***********************************************************************
C                  At all grid points evaluate:                        *
C  1) Channel's width                    B(I)=      b(x)               *
C  2) Channel's Width derivative      BTAG(I)=b'(x)                    *
C  3) Channel's Width derivative/width DBB(I)=b'(x)/b(x)               *
C  4) Channel's bottom slope            S0(I)=(-dz/dx)                 *
C  5) Channel's bottom height above datum Z(K).                        *
C                  For all computational cells evaluate:               *
C  1) Integal of b(x)dx=bdx(k)                                         *
C***********************************************************************

      INCLUDE 'type.inc'
      INCLUDE 'dimen.inc'
      INCLUDE 'vecs.inc'
      INCLUDE 'miscel.inc'

      INTEGER   L,I,K
      REAL*8   S1,S2,S3,S4,S5,SUP,SDN,a0,a1,a2,a3,p0,p,q,poa
      REAL*8   BTAG(LDIM),HTAG(LDIM),H(LDIM)

      LL=L-1
!-----------------------------------------------------------------------
!   Uniform channel-width:                                    KEYwidth=0                                       
!-----------------------------------------------------------------------
      IF (KEYwidth.EQ.0) then
        DO I=2,L                                                          
            B(I)=1.d0                                                                 
          DBB(I)=0.d0                                                                
        END DO                                                                   
        DO  K=2,LL                                                          
          I=K                                                                        
          BDX(K)=X(I+1)-X(I)                                                         
        END DO                                                                   
      END IF
!-----------------------------------------------------------------------
!   Channel-width corresponding to cylindrical symmetry       KEYwidth=1                                                
!-----------------------------------------------------------------------
      IF (KEYwidth.EQ.1) then
        pai=4.d0*atan(1.d0)
          DBB(2)=0.d0                                                                
        DO I=2,L                                                          
                             B(I)=2.d0*pai*X(I)
          IF(X(I).GT.1.d-8) DBB(I)=1.d0/X(I)                                          
        END DO                                                                   
        DO  K=2,LL                                                          
          I=K                                                                        
          BDX(K)=(X(I+1)-X(I))*(X(I+1)+X(I))*pai
!         BDX(K)=(X(I+1)-X(I))*(X(I+1)+X(I))/2.d0                                    
        END DO                                                                   
      END IF

!-----------------------------------------------------------------------
! CHANNEL'S BOTTOM SLOPE So(I)=(-dz/dx) (So(I) is point centered)
!   Z(K)=-integral(So(I)dx)+constant    ( Z(K) is  cell centered)
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!    Flat channel bed:                                     KEYbedslope=0
!-----------------------------------------------------------------------
        IF (KEYbedslope.EQ.0) then
          DO I=2,L                                                          
            So(I)= 0.000d0                                                                
          END DO                                                                   
          DO K=2,LL                                                         
             Z(K)= 0.000d0                                                                
          END DO                                                                   
             Z(1)= 0.d0
             Z(L)= 0.d0
        END IF

!-----------------------------------------------------------------------
!    Constant bottom slope                                 KEYbedslope=1
!-----------------------------------------------------------------------
        IF (KEYbedslope.EQ.1) then     
          DO I=2,L                                                          
            So(I)=+0.001d0                                                                
          END DO                                                                   
          do K=LL,2,-1
            I=K                                                                        
            Z(K)= So(I)*(X(L)-(X(I)+X(I+1))/2.d0) ! Assuming that at X(L) Z=0
            print *, 'K,Z(K)=',K,Z(K)
          end do
!           Z(L)= So(L)*(X(L)-(X(L)+(X(L)-X(L-1))/2.d0))
!           Z(1)= So(2)*(X(L)-(X(2)-(X(3)-X(2  ))/2.d0)) 
            Z(L)= 0.d0
            Z(1)= So(2)* (X(L)-X(1))
            print *, 'L,Z(L)=',L,Z(L)
            print *, '  Z(1)='  ,Z(1)
!-----------------------------------------------------------------------
! Note Z is defined to be cell centred. However, for a bed slope 
! different from 0 the values of Z at the boundaries are needed.
! Hence, only for this 2 points Z is point-centered.
!-----------------------------------------------------------------------
        END IF

!-----------------------------------------------------------------------
!   CHANNEL'S BED FRICTION (Point centered)                KEYfriction=0
!-----------------------------------------------------------------------
!                  NO FRICTION WITH CHANNEL BED
! Note, Sf(I) depends on u(x,t) and h(x,t) and has to be defined
!  while solving for u and h. This entry is only to secure that 
!  SfL(I),SfR(I) and Sf(I) are always defined
!-----------------------------------------------------------------------
        IF (KEYfriction.EQ.0) then
          DO I=2,L                                                          
            SfL(I)= 0.d0                                                                
            SfR(I)= 0.d0                                                                
            Sf (I)= 0.d0                                                                
          END DO                                                                   
        END IF

      GOTO (1,2,3,4,5,6,7,8,9,10), Problem
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C  Dam Break Problem -- Assuming a right-facing shock (dam high on left)
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 1    CONTINUE
      GOTO 100
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 2    CONTINUE
!-----------------------------------------------------------------------
!    MacDonald_1 Rectangular channel eta=0.02 [s/m^.33333]     Problem=2
!                varying width and bed-slope + friction   
!                S.Roberts & C. Zoppou book p. 162-164          
!-----------------------------------------------------------------------
        KEYfriction=1
        KEYbedslope=2
        KEYwidth   =2
          a0= 1.500000d0
          a1=-0.230680d0
          a2= 0.248367d0
          a3=-0.228271d0
        DO I=2,L                                                          
          S1=X(I)/1000.d0
          S2=S1*S1
          S3=S2*S1
          S4=S2*S2
             B(I)=10.d0-64.d0*(     S2-2.d0*S3+     S4)                                            
          BTAG(I)=     -64.d0*(2.d0*S1-6.d0*S2+4.d0*S3)/1000.d0
           DBB(I)=BTAG(I)/B(I)                                                           
          IF(X(I).LE.500.d0) THEN
           H   (I)=-1.d0/40.d0+ 1.d0/(1.d0+2.d0*(S1-0.5d0)**2)
           HTAG(I)=-4.d-3*(S1-.5d0)/(1.d0+2.d0*(S1-0.5d0)**2)**2
          ELSE
           p0=(s1-1.0d0)/4.d0
           p =(1.d0-2.d0*s1)*15.d0
           H   (I)=a0*exp(p0)+a1*exp(p)+a2*exp(2.d0*p)+a3*exp(3.d0*p)
           HTAG(I)=a0/4.d3*exp(p0)-3.d-2*a1*exp(p)-6.d-2*a2*exp(2.d0*p)
     1                                            -9.d-2*a3*exp(3.d0*p)
          END IF
        END DO                                                                   
        DO  K=2,LL                                                          
          I=K                                                                        
          S1=X(I+1)/1000.d0
          S2=S1*S1
          S3=S2*S1
          S4=S2*S2
          S5=S2*S3
          SUP=1.d3*(10.d0*S1-64.d0*(S3/3.d0+s4/2.d0+S5/5.d0))
          S1=X(I  )/1000.d0
          S2=S1*S1
          S3=S2*S1
          S4=S2*S2
          S5=S2*S3
          SDN=1.d3*(10.d0*S1-64.d0*(S3/3.d0+s4/2.d0+S5/5.d0))
          BDX(K)=SUP-SDN
        END DO                                                                   
!    For MacDonald 1 problem So depends on Btag ,Htag and Q so that we define it here
        Q=20       ! [m^3/s]
        eta2=4.d-4 ! [s/m^(-1/3)]^2
        DO I=2,L                                                          
         POA= 2.d0/b(I)+1.d0/H(I)
         So(I)=(1.d0-Q**2/GG/b(I)**2/H(I)**3)*HTAG(I)
     1      +  eta2*Q*abs(Q)/b(I)**2/H(I)**2*POA**(4.d0/3.d0)
     2              -Q**2/GG/b(I)**3/H(I)**2 *BTAG(I)
        END DO                                                                   
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!   I need to find some code for numerical integratio to find z 
!           Z=-integral(So(x)dx) + constant
!           Z( L)= 0.d0
!           Z(LL)= Z(L)+.d0
!           Z(LL)=-So(L)*(X(L)-(X(L)+X(LL))/2.d0) ! Assuming that at X(L) Z=0
!         DO K=LL,2,-1                                                         
!           I=K                                                                        
!           Z(K)= 0.000d0                                                                
!           print *, 'K,Z(K)=',K,Z(K)
!         END DO                                                                   
!            Z(1)= 0.d0
      GOTO 100
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 3    CONTINUE
      GOTO 100
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 4    CONTINUE
      GOTO 100
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 5    CONTINUE
      GOTO 100
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 6    CONTINUE
      GOTO 100
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 7    CONTINUE
      GOTO 100
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 8    CONTINUE
      GOTO 100
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 9    CONTINUE
      GOTO 100
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 10   CONTINUE
      GOTO 100
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 100  CONTINUE



      RETURN                                                                     
C***********************************************************************
C************* ERROR MESSAGES ******************************************
C***********************************************************************
 7001 CONTINUE
      PRINT 7101,KEYSYM 
 7101 FORMAT(/'FROM DUCVAL.  ILLEGAL KEYSYM=',I3 )
      CALL ENDRUN(7001)
      END                                                                        
