      SUBROUTINE FLUXES(L)
C***********************************************************************
C***********************************************************************
C  SOLVE Generalized Riemann Problems at ALL COLUMN GRID POINTS.
C  USE  FLUX--SORTED   LIST_F(I,N)
C***********************************************************************
C***********************************************************************

      INCLUDE 'type.inc'
      INCLUDE 'dimen.inc'
      INCLUDE 'vecs.inc'
      INCLUDE 'miscel.inc'

      INTEGER   L,I,II
      REAL*8    BETA03,BETA0,RO0,C0,DUDALFA,DRODALFA,DUDBETA,DRODBETA,
     1          ALFAp,BETAp,UXmid(LDIM),ROXmid(LDIM)
      REAL*8   beta,b_abeta,b_a1,fac1,fac2,
     1         DUDALFA_ALFAp, DRODALFA_ALFAp

C***********************************************************************
C***********************************************************************
C  STEP 1.  DETERMINATION OF FLOW VARIABLE VALUE IN ONE OF SIX (X,T)
C           SEGMENTS CORRESPONDING TO SOLUTION OF RIEMANN PROBLEM
C***********************************************************************
C  The time-derivatives with constant X are evaluated from the
C  constant-xi time-derivatives using the identity
C   (dQ/dt)x = (dQ/dt)xi -RO*U*(dQ/dxi)
C***********************************************************************
                      DO I=2,L
C-----------------------------------------------------------------------
C  SUBSTEP 1.0  VERIFY  NFLUX(I)  FOR ALL POINTS
C-----------------------------------------------------------------------

      IF(NFLUX(I).LE.0.OR.NFLUX(I).GT.6) THEN
         II=I
         GOTO 7001
      ENDIF

C-----------------------------------------------------------------------
C  SUBSTEP 1.1  CASE  NFLUX=1 
C-----------------------------------------------------------------------

                IF(NFLUX(I).EQ.1) THEN
      Ux    (I) =  UL  (I)
      Rox   (I) =  ROL (I)
      DUDTx (I) = -CL(I)**2*DRODXIL(I) - ROL(I)*UL(I)*DUDXIL(I)
      DUDTx (I) =  DUDTx (I) * b(I) + GG*(So(I)-Sf (I))
      DRODTx(I) = -ROL(I)**2*DUDXIL(I) - ROL(I)*UL(I)*DRODXIL(I)
      DRODTx(I) =  DRODTx(I) * b(I) - UL(I)*ROL(I)*DBB(I) 
                ENDIF ! Nflux=1

C-----------------------------------------------------------------------
C  SUBSTEP 1.2  CASE  NFLUX=6 
C-----------------------------------------------------------------------

                IF(NFLUX(I).EQ.6) THEN
      Ux    (I) =  UR  (I)
      Rox   (I) =  ROR (I)
      DUDTx (I) = -CR(I)**2*DRODXIR(I) - ROR(I)*UR(I)*DUDXIR(I)
      DUDTx (I) =  DUDTx (I) * b(I) + GG*(So(I)-Sf (I)) 
      DRODTx(I) = -ROR(I)**2*DUDXIR(I) - ROR(I)*UR(I)*DRODXIR(I)
      DRODTx(I) = DRODTx(I) * b(I) - UR(I)*ROR(I)*DBB(I)
                ENDIF ! Nflux=6

C-----------------------------------------------------------------------
C  SUBSTEP 1.3  CASE  NFLUX=3
C-----------------------------------------------------------------------

                IF(NFLUX(I).EQ.3) THEN
      Ux    (I) =  USTAR (I)
      ROx   (I) =  ROSTAR(I)
      DUDTx (I) =  DUDT  (I) + USTAR(I)*DRODT(I)/ROSTAR(I)
      DUDTx (I) =  DUDTx (I) + USTAR(I)**2*DBB(I)
      DRODTx(I) =  DRODT (I) + USTAR(I)*DUDT (I)/GG
      DRODTx(I) =  DRODTx(I) - USTAR(I)*(So(I)-Sf (I))
                ENDIF ! Nflux=3

C-----------------------------------------------------------------------
C  SUBSTEP 1.4  CASE  NFLUX=4
C-----------------------------------------------------------------------

                IF(NFLUX(I).EQ.4) THEN
      Ux    (I) =  USTAR (I)
      ROx   (I) =  ROSTAR(I)
      DUDTx (I) =  DUDT  (I) + USTAR(I)*DRODT(I)/ROSTAR(I)
      DUDTx (I) =  DUDTx (I) + USTAR(I)**2*DBB(I)
      DRODTx(I) =  DRODT (I) + USTAR(I)*DUDT (I)/GG
      DRODTx(I) =  DRODTx(I) - USTAR(I)*(So(I)-Sf (I))
                ENDIF ! Nflux=4

C-----------------------------------------------------------------------
C  SUBSTEP 1.5  THE SONIC CASE  NFLUX=2 
C-----------------------------------------------------------------------

                IF(NFLUX(I).EQ.2) THEN
      BETA03    =  (2.d0 + UL(I)/CL(I))/3.d0
      BETA0     =  BETA03**3
      C0        =   CL(I)*BETA03
      RO0       =  ROL(I)*BETA03**2 ! RO/ROL=(G/GL)**(2/3)

C-----------------------------------------------------------------------
C   b*a(   1) eq. (5.25)
C   b*a(beta) eq. (5.45) gamma=2, H(beta)=0 + source terms.
C-----------------------------------------------------------------------
        b_a1   =b(I)*(DUDXIL(I)+CL(I)/ROL(I)*DRODXIL(I))
     1                  -GG/GL(I)*(So(I)-Sf (I))
        b_abeta=b_a1
         if(KEYwidth.ne.0) then
           beta =  BETA0
           fac1 =-( UL(I)+2.d0*CL(I))*(beta**(-1./6.d0)-1.d0)
           fac2 =         4.d0*CL(I) *(beta**(+1./6.d0)-1.d0)
           b_abeta =b_a1 +dbb(I)/ROL(I)*(-fac1+fac2)
         end if

!     DUDALFA   =  DUDXIL(I) + CL(I)*DRODXIL(I)/ROL(I) ! a(beta)=a(1)
!     DRODALFA  =  RO0*DUDALFA/C0
!     ALFAp     = -GL(I)*SQRT(BETA0)
!     To avoid division by b(I), ALFAp DRODALFA and DUDALFA are not computed seperately
!      DUDALFA_ALFAp =  DUDALFA*ALFAp 
!     DRODALFA_ALFAp = DRODALFA*ALFAp

       DUDALFA_ALFAp =  -GL(I)*SQRT(BETA0)*b_abeta
      DRODALFA_ALFAp =                - RO0**2/SQRT(BETA0)*b_abeta
      DRODALFA_ALFAp = DRODALFA_ALFAp- C0/2.d0*(RO0*dbb(I)+So(I)-Sf (I))

!     BETAp     = -BETA0**(7.d0/6.d0)*ROL(I)*DUDALFA/4.d0
      BETAp     = -BETA0**(7.d0/6.d0)*ROL(I)*b_abeta/4.d0
      BETAp     =  BETAp + C0*BETA0/2.d0*(dbb(I) - (So(I)-Sf (I))/RO0)
      DUDBETA   = -2.d0*C0/(3.d0*BETA0)
      DUDBETA   = DUDBETA + dbb(I)*C0/RO0*sqrt(BETA0)
      DUDBETA   = DUDBETA - C0/RO0**2*SQRT(BETA0)*(So(I)-Sf (I))
      DRODBETA  =  2.d0*RO0/(3.d0*BETA0)

      Ux    (I) =  C0 ! Sonic condition  u-c=0
      ROx   (I) =  RO0
!     DUDTx (I) =   DUDALFA*ALFAp +  DUDBETA*BETAp
!     DRODTx(I) =  DRODALFA*ALFAp + DRODBETA*BETAp
      DUDTx (I) =   DUDALFA_ALFAp +  DUDBETA*BETAp
      DRODTx(I) =  DRODALFA_ALFAp + DRODBETA*BETAp
                ENDIF ! Nflux=2

C-----------------------------------------------------------------------
C  SUBSTEP 1.6  THE SONIC CASE  NFLUX=5 
C-----------------------------------------------------------------------

                IF(NFLUX(I).EQ.5) THEN
      BETA03    =  (2.d0 - UR(I)/CR(I))/3.d0
      BETA0     =  BETA03**3
      C0        =   CR(I)*BETA03
      RO0       =  ROR(I)*BETA03**2 ! RO/ROR=(G/GR)**(2/3)

C-----------------------------------------------------------------------
C   b*a(   1) eq. (5.25)
C   b*a(beta) eq. (5.45) gamma=2, H(beta)=0 + source terms.
C-----------------------------------------------------------------------
        b_a1   =b(I)*(DUDXIR(I) - CR(I)/ROR(I)*DRODXIR(I))
     1                       - GG/GR(I)*(So(I)-Sf (I))
        b_abeta=b_a1
         if(KEYwidth.ne.0) then
           beta =  BETA0
           fac1 =-(-UR(I)+2.d0*CR(I))*(beta**(-1./6.d0)-1.d0)
           fac2 =         4.d0*CR(I) *(beta**(+1./6.d0)-1.d0)
           b_abeta =b_a1 - dbb(I)/ROR(I)*(-fac1+fac2)
         end if

!     DUDALFA   =  DUDXIR(I) - CR(I)*DRODXIR(I)/ROR(I) ! a(beta)=a(1)
!     DRODALFA  = -RO0*DUDALFA/C0
!     ALFAp     =  GR(I)*SQRT(BETA0)
!     To avoid division by b(I), ALFAp DRODALFA and DUDALFA are not computed seperately
!      DUDALFA_ALFAp =  DUDALFA*ALFAp 
!     DRODALFA_ALFAp = DRODALFA*ALFAp

       DUDALFA_ALFAp =  -GR(I)*SQRT(BETA0)*b_abeta
      DRODALFA_ALFAp =                - RO0**2/SQRT(BETA0)*b_abeta
      DRODALFA_ALFAp = DRODALFA_ALFAp- C0/2.d0*(RO0*dbb(I)+So(I)-Sf (I))


!     BETAp     = -BETA0**(7.d0/6.d0)*ROR(I)*DUDALFA/4.d0
      BETAp     = -BETA0**(7.d0/6.d0)*ROR(I)*b_abeta/4.d0
      BETAp     =  BETAp + C0*BETA0/2.d0*(dbb(I) - (So(I)-Sf (I))/RO0)
!     DUDBETA   =  2.d0*C0/(3.d0*BETA0)
!     DRODBETA  =  2.d0*RO0/(3.d0*BETA0)
      DUDBETA   = -2.d0*C0/(3.d0*BETA0)
      DUDBETA   = DUDBETA + dbb(I)*C0/RO0*sqrt(BETA0)
      DUDBETA   = DUDBETA - C0/RO0**2*SQRT(BETA0)*(So(I)-Sf (I))
      DRODBETA  =  2.d0*RO0/(3.d0*BETA0)

      Ux    (I) = -C0 ! Sonic condition  u+c=0
      ROx   (I) =  RO0
!     DUDTx (I) =   DUDALFA*ALFAp +  DUDBETA*BETAp
!     DRODTx(I) =  DRODALFA*ALFAp + DRODBETA*BETAp
      DUDTx (I) =   DUDALFA_ALFAp +  DUDBETA*BETAp
      DRODTx(I) =  DRODALFA_ALFAp + DRODBETA*BETAp
                ENDIF ! Nflux=5
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
                      END DO ! I=2,L

C***********************************************************************
C***********************************************************************
C  STEP 2.  FLUXES AT TIME  T(N)
C           Note:  Fluxes here are defined from mid-step values of
C                  primitive variables, as in Eq. (5.107) of GRP book.
C                  In other GRP codes we evaluated the time derivative
C                  of flux components, and thence the mid-step fluxes.
C***********************************************************************
C***********************************************************************
C-----------------------------------------------------------------------
C  SUBSTEP 2.1  Mid-Step values.  At  T=T(N+1/2)
C-----------------------------------------------------------------------
      DO 21 I=2,L
          UXmid(I) =  Ux(I) + DT2* DUDTx(I)
         ROXmid(I) = ROx(I) + DT2*DRODTx(I)
 21   CONTINUE
C-----------------------------------------------------------------------
C  SUBSTEP 2.2  MASS AND MOMENTUM FLUXES
C-----------------------------------------------------------------------
      DO 22 I=2,L
!        FLUX(I,1) = ROXmid(I)*UXmid(I)
!        FLUX(I,2) = ROXmid(I)*UXmid(I)**2 + A*ROXmid(I)**2
         FLUX(I,1) = ROXmid(I)*UXmid(I)    *b(I)
         FLUX(I,2) = ROXmid(I)*UXmid(I)**2 *b(I)
         GLUX(I,1) = 0.0d0
         GLUX(I,2) = A*ROXmid(I)**2
 22   CONTINUE

      RETURN
C***********************************************************************
C  ********   ERROR  MESSAGES   ********************
C***********************************************************************
 7001 CONTINUE
      I=II
      PRINT 7101,I,NFLUX(I),UL(I),ROL(I),UR(I),ROR(I),USTAR(I),ROSTAR(I)
     1            ,UW(1,I),UW(2,I),UW(3,I),UW(4,I),UW(5,I)
 7101 FORMAT(/'FLUXES.  RP WAS NOT COMPUTED, SINCE NO NFLUX DEFINED'/
     1        'I,NFLUX(I)        =',   2I6/
     1        'UL,ROL,UR,ROR     =',1P,4E12.4/
     1        'USTAR(I),ROSTAR(I)=',   2E12.4/
     2        'UW(1,I),UW(2,I)   =',   2E12.4/
     2        'UW(3,I),UW(4,I)   =',   2E12.4/
     2        'UW(5,I)           =',   1E12.4//)
      CALL ENDRUN(7001)
      END
