      SUBROUTINE GRP(L)

                include  'type.inc'
                include  'dimen.inc'
                include  'vecs.inc'
                include  'miscel.inc'

      INTEGER  L,I,N
      REAL*8   ETAL,ETAR,Lu,Lro,Llmd,Lso
      REAL*8   AAL,BBL,DDL,AAR,BBR,DDR,DET
      REAL*8   beta,b_abeta,b_a1,fac1,fac2
      INTEGER  J

      LOGICAL ACSTL,ACSTR

C***********************************************************************
C  STEP 1.  ACOUSTIC SOLUTION TO GENERALIZED RIEMANN PROBLEM.
C           GRP IS SOLVED IN THE ACOUSTIC APPROXIMATION AT ALL POINTS
C***********************************************************************

C=======================================================================
C   1.1   Preliminary arrays
C=======================================================================

      DO I=2,L
           WL(I)     = -GL(I)
           WR(I)     =  GR(I)
           DUDROL(I) = -(CL(I)/ROL(I))
           DUDROR(I) =  (CR(I)/ROR(I))
           NFLUX (I) =  0
         ACOUSTIC(I) = .FALSE.
             UW(1,I) =  1.d77
             UW(2,I) =  1.d77
             UW(3,I) =  1.d77
             UW(4,I) =  1.d77
             UW(5,I) =  1.d77
      END DO

      IF(EPS_acoust.LE.0.d0) GOTO 111 ! No acoustic approximation

C=======================================================================
C   1.2   Intersection (ROSTAR,USTAR) of linear curves in (RO,U)
C=======================================================================
C         U = UL + DUDROL*(RO-ROL) 
C         U = UR + DUDROR*(RO-ROR) 
C=======================================================================

      DO I=2,L
        ROSTAR(I)=( UL(I)- UR(I)+ROR(I)*DUDROR(I)-ROL(I)*DUDROL(I))/
     1            (  DUDROR(I)-  DUDROL(I))
         USTAR(I)=(ROL(I)-ROR(I)+ UR(I)/DUDROR(I)- UL(I)/DUDROL(I))/
     1            (1/DUDROR(I)-1/DUDROL(I))
      END DO

C=======================================================================
C   1.3   Star wave speeds
C=======================================================================

      DO I=2,L
         CSTAR(I)=SQRT(GG*ROSTAR(I))
      END DO

      DO I=2,L
         GSTAR(I)=ROSTAR(I)*CSTAR(I)
      END DO

C=======================================================================
C   1.4   Linear curves for DU/DT, DRO/DT
C=======================================================================
C         AAL*DUDT + BBL*DRODT = DDL
C         AAR*DUDT + BBR*DRODT = DDR
C         Eq. (5.13) in book + extra terms for slope and friction
C=======================================================================

      DO I=2,L
         AAL =  1.d0
         BBL =  CSTAR(I)/ROSTAR(I) ! multiplied by CSTAR**2 for DRO/DT
C=======================================================================
!        DDL = (-GL(I)*DUDXIL(I)-CL(I)**2*DRODXIL(I))*
!    1              (GL(I)/GSTAR(I)) ! according to eq. (5.13) in book.
C=======================================================================
         DDL =((-GL(I)*DUDXIL(I)-CL(I)**2*DRODXIL(I))*b(I)
     1         - dbb(I)*CL(I)*UL(I))*(GL(I)/GSTAR(I))
     2         + GG*(So(I)-Sf (I))
         AAR = -1.d0
         BBR =  CSTAR(I)/ROSTAR(I) ! multiplied by CSTAR**2 for DRO/DT
C=======================================================================
!        DDR = (-GR(I)*DUDXIR(I)+CR(I)**2*DRODXIR(I))*
!    1              (GR(I)/GSTAR(I)) ! according to (5.13) in book.
C=======================================================================
         DDR =((-GR(I)*DUDXIR(I)+CR(I)**2*DRODXIR(I))*b(I)
     1         - dbb(I)*CR(I)*UR(I))*(GR(I)/GSTAR(I))
     2         - GG*(So(I)-Sf (I))
         DET=AAL*BBR-AAR*BBL
          DUDT(I)=(DDL*BBR-DDR*BBL)/DET
         DRODT(I)=(DDR*AAL-DDL*AAR)/DET
      END DO


      DO I=2,L
         ACSTL=ABS(ROSTAR(I)-ROL(I)).LT.EPS_acoust*ROL(I)
         ACSTR=ABS(ROSTAR(I)-ROR(I)).LT.EPS_acoust*ROR(I)
         ACOUSTIC(I)=ACSTL.AND.ACSTR
      END DO

      DO I=2,L
                IF(ACOUSTIC(I)) THEN
         UW(1,I)=UL(I)-CL(I)
         UW(2,I)=UL(I)-CL(I)
         UW(3,I)=USTAR(I)
         UW(4,I)=UR(I)+CR(I)
         UW(5,I)=UR(I)+CR(I)
                ENDIF
      END DO

      DO I=2,L
                IF(ACOUSTIC(I)) THEN
         DO 11 N=1,5
         NFLUX(I)=N
         IF (UW(N,I).GE.UG(I)) GO TO 12
 11      CONTINUE
         NFLUX(I)=6
 12      CONTINUE
                ENDIF
      END DO


 111  CONTINUE

C***********************************************************************
C  STEP 2.  EXACT SOLUTION TO RIEMANN PROBLEM
C***********************************************************************

      DO 20 I=2,L
                  IF(.NOT.ACOUSTIC(I)) THEN
      CALL RIEMANN(L,I,MATL(I),UL(I),ROL(I),CL(I),GL(I),
     1                         UR(I),ROR(I),CR(I),GR(I),
     2                         USTAR(I),ROSTAR(I),CSTAR(I),GSTAR(I),
     3                         WL(I),WR(I),UW(1,I),
     4                         ACOUSTIC(I),SHOCKL(I),SHOCKR(I))

                  ENDIF ! .NOT.ACOUSTIC(I)
 20   CONTINUE

C***********************************************************************
C  STEP 3.  EXACT SOLUTION TO GENERALIZED RIEMANN PROBLEM:
C           TIME-DERIVATIVES ON CONTACT SURFACE
C***********************************************************************
C=======================================================================
C         AAL*DUDT + BBL*DRODT = DDL
C         AAR*DUDT + BBR*DRODT = DDR
C=======================================================================

      DO 30 I=2,L
                  IF(.NOT.ACOUSTIC(I)) THEN
      ETAL   = ROSTAR(I)/ROL(I)

      IF(.NOT.SHOCKL(I)) THEN  
C=======================================================================
!                          LEFT CRW
C=======================================================================
         AAL =  1.d0
         BBL =  CSTAR(I)/ROSTAR(I)
C=======================================================================
!        DDL for constant width and no source terms.
!        DDL = (-GL(I)*DUDXIL(I)-CL(I)**2*DRODXIL(I))*
!    1          SQRT(GSTAR(I)/GL(I))
!     Note power 1/2 different from acoustic case. (5.48) in book
C=======================================================================
C-----------------------------------------------------------------------
C   b*a(   1) eq. (5.25)
C   b*a(beta) eq. (5.45) gamma=2, H(beta)=0 + source terms
C-----------------------------------------------------------------------
         b_a1=b(I)*(DUDXIL(I)+CL(I)/ROL(I)*DRODXIL(I))
     1                  -GG/GL(I)*(So(I)-Sf (I))
         b_abeta =b_a1
         if(KEYwidth.ne.0) then 
           beta =  GSTAR(I)/ GL(I)
           fac1 =-( UL(I)+2.d0*CL(I))*(beta**(-1./6.d0)-1.d0)
           fac2 =         4.d0*CL(I) *(beta**(+1./6.d0)-1.d0)
           b_abeta =b_a1 + dbb(I)/ROL(I)*(-fac1+fac2)
         end if
C-----------------------------------------------------------------------
! dl=-sqrt(gl*gstar)*b*a(betastar)-lambda*ustar*cstar [Eq. (5.48) in book]
!  Note: this expression stays the same even for nonzero So or Sf
C-----------------------------------------------------------------------
         DDL =-SQRT(GSTAR(I)*GL(I))*b_abeta-dbb(I)*CSTAR(I)*USTAR(I)
      ELSE                   
C=======================================================================
!                            LEFT SHOCK. 
C=======================================================================
         AAL = -1.5d0 - 0.25d0/ETAL - 0.25d0/ETAL**2
!        Analysis for DRO/DT. No CSTAR**2 factor for BBL
         BBL = -SQRT(A/ROL(I))*(ETAL+1.5d0+1.5d0/ETAL)/
     1                     SQRT(ETAL*(1+ETAL))
         Lu  =  GL(I)*(2+3*ETAL+3*ETAL**2)/SQRT(8*ETAL*(1+ETAL))
         Lro =  0.25d0*CL(I)**2*(6+ETAL+ETAL**2)
         Lu  =  Lu *b(I)
         Lro =  Lro*b(I)
         DDL =  Lu*DUDXIL(I) + Lro*DRODXIL(I)
! Check  sign (the expression were derived for a right shock)
         Llmd= sqrt(A*ROL(I)/ETAL/(1.d0+ETAL))
         Llmd= Llmd*((1+ETAL)*USTAR(I)-UL(I)/2.d0*(2.d0+ETAL**2+ETAL))
         Lso = 1.d0+0.25d0*(2.d0+1.d0/ETAL+1.d0/ETAL**2)
         DDL =  DDL - Llmd*DBB(I) - Lso*GG*(So(I)-Sf (I))
      ENDIF                   ! LEFT CRW/SHOCK

      ETAR = ROSTAR(I)/ROR(I)
      IF(.NOT.SHOCKR(I)) THEN
C=======================================================================
!                          RIGHT CRW
C=======================================================================
         AAR = -1.d0
         BBR =  CSTAR(I)/ROSTAR(I)
C-----------------------------------------------------------------------
!        DDR for constant width and no source terms.
!        DDR = (-GR(I)*DUDXIR(I)+CR(I)**2*DRODXIR(I))*
!    1          SQRT(GSTAR(I)/GR(I)) 
!            Note power 1/2 different from acoustic case. (5.48) in book
C-----------------------------------------------------------------------
C   b*a(   1) eq. (5.25)
C   b*a(beta) eq. (5.45) gamma=2, H(beta)=0 + source terms
C-----------------------------------------------------------------------
         b_a1=b(I)*(DUDXIR(I)-CR(I)/ROR(I)*DRODXIR(I))
     1                  +GG/GR(I)*(So(I)-Sf (I))
         b_abeta =b_a1
         if(KEYwidth.ne.0) then 
           beta =  GSTAR(I)/ GR(I)
           fac1 =-(-UR(I)+2.d0*CR(I))*(beta**(-1./6.d0)-1.d0)
           fac2 =         4.d0*CR(I) *(beta**(+1./6.d0)-1.d0)
           b_abeta =b_a1 - dbb(I)/ROR(I)*(-fac1+fac2)
         end if
C-----------------------------------------------------------------------
!  dr=-sqrt(gr*gstar)*b*a(beta)-lambda*ustar*cstar eq. 5.48 in book
!  Note: this expression stays the same even for nonzero So or Sf
C-----------------------------------------------------------------------

         DDR =-SQRT(GSTAR(I)*GR(I))*b_abeta-dbb(I)*CSTAR(I)*USTAR(I)

!------------------     Debugging  -------------------------------------
!     if (abs(ddr-ddr1).gt.1.d-7) then
!         print *,'ddr,ddr1', ddr,ddr1
!         print *,'b_a1,b_abeta', b_a1,b_abeta
!         print *,'b(J)        ', (  b(J), J=1,100)
!         print *,'dbb(J)      ', (dbb(J), J=1,100)
!         print *,'        beta'      ,   beta
!         print *,'I, dbb(I),So(I)',I, dbb(I),So(I)
!         call endrun(701)
!     end if
!-----------------------------------------------------------------------
      ELSE
C=======================================================================
!                           RIGHT SHOCK. 
!               Analysis for DRO/DT. No CSTAR**2 factor for BBR
C=======================================================================
         AAR =  1.5d0 + 0.25d0/ETAR + 0.25d0/ETAR**2
         BBR = -SQRT(A/ROR(I))*(ETAR+1.5d0+1.5d0/ETAR)/
     1                     SQRT(ETAR*(1+ETAR))
         Lu  =  GR(I)*(2+3*ETAR+3*ETAR**2)/SQRT(8*ETAR*(1+ETAR))
         Lro = -0.25d0*CR(I)**2*(6+ETAR+ETAR**2)
         Lu  =  Lu *b(I)
         Lro =  Lro*b(I)
         DDR =  Lu*DUDXIR(I) + Lro*DRODXIR(I)
         Llmd= sqrt(A*ROR(I)/ETAR/(1.d0+ETAR))
         Llmd= Llmd*((1+ETAR)*USTAR(I)+UR(I)/2.d0*(2.d0+ETAR**2+ETAR))
         Lso = 1.d0+0.25d0*(2.d0+1.d0/ETAR+1.d0/ETAR**2)
         DDR =  DDR + Llmd*DBB(I) + Lso*GG*(So(I)-Sf (I))
      ENDIF                   ! RIGHT CRW/SHOCK

      DET=AAL*BBR-AAR*BBL
       DUDT(I)=(DDL*BBR-DDR*BBL)/DET
      DRODT(I)=(DDR*AAL-DDL*AAR)/DET
                  ENDIF ! .NOT.ACOUSTIC(I)

 30   CONTINUE

C***********************************************************************
C  STEP 4.  FLUX INDEX  NFLUX(I)  ACCORDING TO ASSOCIATED RP SOLUTION
C***********************************************************************

      DO 40 I=2,L
                  IF(.NOT.ACOUSTIC(I)) THEN
         DO 41 N=1,5
         NFLUX(I)=N
         IF (UW(N,I).GE.UG(I)) GO TO 42
 41      CONTINUE
         NFLUX(I)=6
 42      CONTINUE
                  ENDIF ! .NOT.ACOUSTIC(I)

 40   CONTINUE

      RETURN
C************* ERROR MESSAGES ******************************************
 7001 CONTINUE
      CALL ENDRUN(701)
      END

